@@ -1,99 +1,91 @@
-nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
-                  mom = NULL,
-                  exact = NULL,
-                  near_exact = NULL,
-                  fine = NULL,
-                  near_fine = NULL,
-                  near = NULL,
-                  far = NULL,
-                  solver = NULL) {
-  
+nmatch <- function(dist_mat, subset_weight = NULL, total_pairs = NULL,
+                   mom = NULL,
+                   exact = NULL,
+                   near_exact = NULL,
+                   fine = NULL,
+                   near_fine = NULL,
+                   near = NULL,
+                   far = NULL,
+                   solver = NULL) {
+
   if (is.null(mom)) {
-    mom_covs = NULL
-    mom_tols = NULL
-    mom_targets = NULL
-  } else {
-    mom_covs = mom$covs
-    mom_tols = mom$tols
-    mom_targets = mom$targets
+    mom <- list()
   }
-  
+
+  mom_covs <- mom$covs
+  mom_tols <- mom$tols
+  mom_targets <- mom$targets
+
   if (is.null(exact)) {
-    exact_covs = NULL
-  } else {
-    exact_covs = exact$covs
+    exact <- list()
   }
-  
+
+  exact_covs <- exact$covs
+
   if (is.null(near_exact)) {
-    near_exact_covs = NULL
-    near_exact_devs = NULL
-  } else {
-    near_exact_covs = near_exact$covs
-    near_exact_devs = near_exact$devs
+    near_exact <- list()
   }
-  
+
+  near_exact_covs <- near_exact$covs
+  near_exact_devs <- near_exact$devs
+
   if (is.null(fine)) {
-    fine_covs = NULL
-  } else {
-    fine_covs = fine$covs
+    fine <- list()
   }
-  
+
+  fine_covs <- fine$covs
+
   if (is.null(near_fine)) {
-    near_fine_covs = NULL
-    near_fine_devs = NULL
-  } else {
-    near_fine_covs = near_fine$covs
-    near_fine_devs = near_fine$devs
+    near_fine <- list()
   }
-  
+
+  near_fine_covs <- near_fine$covs
+  near_fine_devs <- near_fine$devs
+
   if (is.null(near)) {
-    near_covs = NULL
-    near_pairs = NULL
-    near_groups = NULL
-  } else {
-    near_covs = near$covs
-    near_pairs = near$pairs
-    near_groups = near$groups
+    near <- list()
   }
-  
+
+  near_covs <- near$covs
+  near_pairs <- near$pairs
+  near_groups <- near$groups
+
   if (is.null(far)) {
-    far_covs = NULL
-    far_pairs = NULL
-    far_groups = NULL
-  } else {
-    far_covs = far$covs
-    far_pairs = far$pairs
-    far_groups = far$groups
+    far <- list()
   }
-  
+
+  far_covs <- far$covs
+  far_pairs <- far$pairs
+  far_groups <- far$groups
+
   if (is.null(solver)) {
-    t_max = 60 * 15
-    approximate = 1
-    solver = "highs"
-  } else {
-    t_max = solver$t_max
-    approximate = solver$approximate
-    trace = solver$trace
-    round_cplex = solver$round_cplex
-    solver = solver$name
+    solver <- list(t_max = 60 * 15,
+                   approximate = 1,
+                   name = "highs")
   }
-  
-  cat(format("  Building the matching problem..."), "\n")
-  
-  #! Total number of units
-  n_tot = nrow(dist_mat)
-  
-  #! Total number of decision variables
-  n_dec = (n_tot*(n_tot-1))-sum(1:(n_tot-1))
-  
+
+  t_max <- solver$t_max
+  approximate <- solver$approximate
+  trace <- solver$trace
+  round_cplex <- solver$round_cplex
+  solver <- solver$name
+
   if (is.null(subset_weight)) {
     subset_weight = 0
   }
-  
-  #! cvec  
+
+  message("  Building the matching problem...")
+
+  #! Total number of units
+  n_tot = nrow(dist_mat)
+
+  #! Total number of decision variables
+  n_dec = (n_tot*(n_tot-1))-sum(1:(n_tot-1))
+
+  #! cvec
   cvec = t(dist_mat)[lower.tri(dist_mat)]-(subset_weight*rep(1, n_dec))
-  
-  #! Amat		
+
+  #! Amat
   #rows_far_all = NULL
   #cols_far_all = NULL
   #vals_far_all = NULL
@@ -133,7 +125,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   rows_target = NULL
   cols_target = NULL
   vals_target = NULL
-  
+
   #! Nonbipartite matching constraints
   rows_nbm = sort(rep(1:n_tot, n_tot-1))
   temp = matrix(0, nrow = n_tot, ncol = n_tot)
@@ -144,7 +136,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   cols_nbm = cols_nbm[!is.na(cols_nbm)]
   vals_nbm = rep(1, (n_tot-1)*n_tot)
   row_count = max(rows_nbm)
-  
+
   #! Far constraints
   rows_ind_far_pairs = list()
   if (!is.null(far_covs)) {
@@ -157,12 +149,12 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
       #! Far on average constraints
       if (!is.null(far_groups)) {
         far_group = far_groups[j]
-        row_ind_far_all = rep(row_count+1, n_dec)  	
-        col_ind_far_all = rep(1:n_dec, 1)			
+        row_ind_far_all = rep(row_count+1, n_dec)
+        col_ind_far_all = rep(1:n_dec, 1)
         i_ind = rep(1:(n_tot-1), (n_tot-1):1)
         aux = matrix(rep(1:n_tot, n_tot), nrow = n_tot, byrow = F)
         j_ind = aux[lower.tri(aux)]
-        vals_far_all = 	far_cov[i_ind]-far_cov[j_ind]-(far_group*rep(1, n_dec))	
+        vals_far_all = 	far_cov[i_ind]-far_cov[j_ind]-(far_group*rep(1, n_dec))
         row_count	= max(row_ind_far_all)
       }
       #! Far on all pairs constraints
@@ -171,41 +163,41 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
         aux = abs(outer(far_cov, far_cov, FUN = "-"))
         temp = as.vector(matrix(t(aux)[lower.tri(aux)], nrow = 1, byrow = TRUE))
         cols_ind_far_pairs = which(temp<far_pair)
-        if (length(cols_ind_far_pairs)>0) {
+        if (length(cols_ind_far_pairs) > 0) {
           rows_ind_far_pairs[[j]] = row_count+(1:length(cols_ind_far_pairs))
           vals_far_pairs = rep(1, length(cols_ind_far_pairs))
           row_count	= max(rows_ind_far_pairs[[j]])
         }
-        if (length(cols_ind_far_pairs)==0) {
+        else {
           cols_ind_far_pairs = NULL
           rows_ind_far_pairs[[j]] = -1
           vals_far_pairs = NULL
         }
-      }		
+      }
       #! Put together
       if (!is.null(far_groups) && is.null(far_pairs)) {
         rows_far = c(rows_far, row_ind_far_all)
         cols_far = c(cols_far, col_ind_far_all)
         vals_far = c(vals_far, vals_far_all)
       }
-      if (is.null(far_groups) && !is.null(far_pairs) && all(rows_ind_far_pairs[[j]] != -1)) {
+      else if (is.null(far_groups) && !is.null(far_pairs) && all(rows_ind_far_pairs[[j]] != -1)) {
         rows_far = c(rows_far, rows_ind_far_pairs[[j]])
         cols_far = c(cols_far, cols_ind_far_pairs)
         vals_far = c(vals_far, vals_far_pairs)
       }
-      if (!is.null(far_groups) && !is.null(far_pairs) && all(rows_ind_far_pairs[[j]] != -1)) {
+      else if (!is.null(far_groups) && !is.null(far_pairs) && all(rows_ind_far_pairs[[j]] != -1)) {
         rows_far = c(rows_far, row_ind_far_all, rows_ind_far_pairs[[j]])
         cols_far = c(cols_far, col_ind_far_all, cols_ind_far_pairs)
         vals_far = c(vals_far, vals_far_all, vals_far_pairs)
       }
-      if (!is.null(far_groups) && !is.null(far_pairs) && all(rows_ind_far_pairs[[j]] == -1)) {
+      else if (!is.null(far_groups) && !is.null(far_pairs) && all(rows_ind_far_pairs[[j]] == -1)) {
         rows_far = c(rows_far, row_ind_far_all)
         cols_far = c(cols_far, col_ind_far_all)
         vals_far = c(vals_far, vals_far_all)
       }
     }
   }
-  
+
   #! Far on average constraints
   #if (!is.null(far_covs)) {
   #  rows_far_all = rep(row_count+1, n_dec)
@@ -216,7 +208,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   #  vals_far_all = far_covs[i_ind]-far_covs[j_ind]-(far_groups*rep(1, n_dec))
   #  row_count = max(rows_far_all)
   #}
-  
+
   #! Far on all pairs constraints
   #if (!is.null(far_covs)) {
   #  aux = abs(outer(far_covs, far_covs, FUN = "-"))
@@ -235,9 +227,9 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   #    rows_far_pairs = c(rows_far_pairs, row_count+(1:length(which(temp<far_pairs[i]))))
   #    vals_far_pairs = c(vals_far_pairs, rep(1, length(which(temp<far_pairs[i]))))
   #    row_count = max(rows_far_pairs)
-  #  }  
+  #  }
   #}
-  
+
   #! Near constraints
   rows_ind_near_pairs = list()
   if (!is.null(near_covs)) {
@@ -250,12 +242,12 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
       #! Near on average constraints
       if (!is.null(near_groups)) {
         near_group = near_groups[j]
-        row_ind_near_all = rep(row_count+1, n_dec)    
-        col_ind_near_all = rep(1:n_dec, 1)			
+        row_ind_near_all = rep(row_count+1, n_dec)
+        col_ind_near_all = rep(1:n_dec, 1)
         i_ind = rep(1:(n_tot-1), (n_tot-1):1)
         aux = matrix(rep(1:n_tot, n_tot), nrow = n_tot, byrow = F)
         j_ind = aux[lower.tri(aux)]
-        vals_near_all = 	near_cov[i_ind]-near_cov[j_ind]-(near_group*rep(1, n_dec))	
+        vals_near_all = 	near_cov[i_ind]-near_cov[j_ind]-(near_group*rep(1, n_dec))
         row_count	= max(row_ind_near_all)
       }
       #! Near on all pairs constraints
@@ -264,41 +256,41 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
         aux = abs(outer(near_cov, near_cov, FUN = "-"))
         temp = as.vector(matrix(t(aux)[lower.tri(aux)], nrow = 1, byrow = TRUE))
         cols_ind_near_pairs = which(temp>near_pair)
-        if (length(cols_ind_near_pairs)>0) {
+        if (length(cols_ind_near_pairs) > 0) {
           rows_ind_near_pairs[[j]] = row_count+(1:length(cols_ind_near_pairs))
           vals_near_pairs = rep(1, length(cols_ind_near_pairs))
           row_count	= max(rows_ind_near_pairs[[j]])
         }
-        if (length(cols_ind_near_pairs)==0) {
+        else {
           cols_ind_near_pairs = NULL
           rows_ind_near_pairs[[j]] = -1
           vals_near_pairs = NULL
         }
-      }		
+      }
       #! Put together
       if (!is.null(near_groups) && is.null(near_pairs)) {
         rows_near = c(rows_near, row_ind_near_all)
         cols_near = c(cols_near, col_ind_near_all)
         vals_near = c(vals_near, vals_near_all)
       }
-      if (is.null(near_groups) && !is.null(near_pairs) && all(rows_ind_near_pairs[[j]] != -1)) {
+      else if (is.null(near_groups) && !is.null(near_pairs) && all(rows_ind_near_pairs[[j]] != -1)) {
         rows_near = c(rows_near, rows_ind_near_pairs[[j]])
         cols_near = c(cols_near, cols_ind_near_pairs)
         vals_near = c(vals_near, vals_near_pairs)
       }
-      if (!is.null(near_groups) && !is.null(near_pairs) && all(rows_ind_near_pairs[[j]] != -1)) {
+      else if (!is.null(near_groups) && !is.null(near_pairs) && all(rows_ind_near_pairs[[j]] != -1)) {
         rows_near = c(rows_near, row_ind_near_all, rows_ind_near_pairs[[j]])
         cols_near = c(cols_near, col_ind_near_all, cols_ind_near_pairs)
         vals_near = c(vals_near, vals_near_all, vals_near_pairs)
       }
-      if (!is.null(near_groups) && !is.null(near_pairs) && all(rows_ind_near_pairs[[j]] == -1)) {
+      else if (!is.null(near_groups) && !is.null(near_pairs) && all(rows_ind_near_pairs[[j]] == -1)) {
         rows_near = c(rows_near, row_ind_near_all)
         cols_near = c(cols_near, col_ind_near_all)
         vals_near = c(vals_near, vals_near_all)
       }
     }
   }
-  
+
   #! Near on average constraints
   #if (!is.null(near_covs)) {
   #  rows_near_all = rep(row_count+1, n_dec)
@@ -309,7 +301,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   #  vals_near_all = near_covs[i_ind]-near_covs[j_ind]-(near_groups*rep(1, n_dec))
   #  row_count = max(rows_near_all)
   #}
-  
+
   #! Near on all pairs constraints
   #if (!is.null(near_covs)) {
   #  aux = abs(outer(near_covs, near_covs, FUN = "-"))
@@ -328,11 +320,11 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   #    rows_near_pairs = c(rows_near_pairs, row_count+(1:length(which(temp<near_pairs[i]))))
   #    vals_near_pairs = c(vals_near_pairs, rep(1, length(which(temp<near_pairs[i]))))
   #    row_count = max(rows_near_pairs)
-  #  }  
+  #  }
   #}
-  
+
   #! Moment constraints
-  if (!is.null(mom_covs) & is.null(mom_targets)) {
+  if (!is.null(mom_covs) && is.null(mom_targets)) {
     rows_mom_1 = NA
     cols_mom_1 = NA
     vals_mom_1 = NA
@@ -370,9 +362,9 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
     vals_mom = c(vals_mom_1, vals_mom_2)
     row_count = max(rows_mom)
   }
-  
+
   #! Moment Constraints Target
-  if (!is.null(mom_covs) & !is.null(mom_targets)) {
+  if (!is.null(mom_covs) && !is.null(mom_targets)) {
     n_covs_m = ncol(mom_covs)
     rows_target = sort(rep(1:(4*n_covs_m)+row_count, n_dec))
     for (i in 1:n_covs_m) {
@@ -388,7 +380,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
     }
     row_count = max(rows_target)
   }
-  
+
   #! Exact matching constraints
   rows_exact = numeric()
   cols_exact = numeric()
@@ -402,8 +394,8 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
       vals_exact = c(vals_exact, dist_exact_cov[lower.tri(dist_exact_cov)])
     }
     row_count	= max(rows_exact)
-  }	
-  
+  }
+
   #! Near-exact matching constraints
   rows_near_exact = numeric()
   cols_near_exact = numeric()
@@ -418,7 +410,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
     }
     row_count	= max(rows_near_exact)
   }
-  
+
   #! Fine balance constraints
   if (!is.null(fine_covs)) {
     #! Transform fine_covs to a matrix of binary inds. for each cat. of each fine balancing covariate
@@ -450,7 +442,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
     }
     row_count = max(rows_fine)
   }
-  
+
   #! Near fine balance constraints
   if (!is.null(near_fine_covs)) {
     #! Transform near_fine_covs to a matrix of binary inds. for each cat. of each fine balancing covariate
@@ -468,7 +460,7 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
     n_near_fine_cats = ncol(near_fine_covs_2)
     j = 1
     for (i in 1:n_near_fine_cats) {
-      for(h in 1:2) {
+      for (h in 1:2) {
         rows_near_fine = c(rows_near_fine, rep(row_count+j, n_dec))
         cols_near_fine = c(cols_near_fine, 1:n_dec)
         dist_near_fine_cov = outer(near_fine_covs_2[, i], near_fine_covs_2[, i], "-")
@@ -484,32 +476,32 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
     }
     row_count = max(rows_near_fine)
   }
-  
+
   #! n constraints
   if (!is.null(total_pairs)) {
     rows_n = rep(row_count+1, n_dec)
     cols_n = 1:n_dec
     vals_n = rep(1, n_dec)
     row_count = max(rows_n)
-  }				
-  
+  }
+
   #! Put together
   rows = c(rows_nbm, rows_far, rows_near, rows_mom,
            rows_target, rows_exact, rows_near_exact, rows_fine, rows_near_fine, rows_n)
-  cols = c(cols_nbm, cols_far, cols_near, cols_mom, 
+  cols = c(cols_nbm, cols_far, cols_near, cols_mom,
            cols_target, cols_exact, cols_near_exact, cols_fine, cols_near_fine, cols_n)
-  vals = c(vals_nbm, vals_far, vals_near, vals_mom, 
+  vals = c(vals_nbm, vals_far, vals_near, vals_mom,
            vals_target, vals_exact, vals_near_exact, vals_fine, vals_near_fine, vals_n)
   aux = cbind(rows, cols, vals)[order(cols), ]
   aux = aux[(aux[, 3] != 0), ]
   cnstrn_mat = simple_triplet_matrix(i = aux[, 1], j = aux[, 2], v = aux[, 3])
   Amat = cnstrn_mat
-  
+
   #! bvec
-  bvec = rep(1,  length(table(rows_nbm)))
+  bvec = rep(1, length(table(rows_nbm)))
   #bvec = c(bvec, rep(0, length(table(rows_far_all))))
   #bvec = c(bvec, rep(0, length(table(rows_far_pairs))))
-  
+
   if (!is.null(far_covs)) {
     n_far_covs = ncol(far_covs)
     for (j in 1:n_far_covs) {
@@ -521,10 +513,10 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
       }
     }
   }
-  
+
   #bvec = c(bvec, rep(0, length(table(rows_near_all))))
   #bvec = c(bvec, rep(0, length(table(rows_near_pairs))))
-  
+
   if (!is.null(near_covs)) {
     n_near_covs = ncol(near_covs)
     for (j in 1:n_near_covs) {
@@ -536,11 +528,11 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
       }
     }
   }
-  
+
   bvec = c(bvec, rep(0, length(table(rows_mom))))
-  
+
   bvec = c(bvec, rep(0, length(table(rows_target))))
-  
+
   if (!is.null(exact_covs)) {
     bvec = c(bvec, rep(0, ncol(exact_covs)))
   }
@@ -559,47 +551,47 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   if (!is.null(total_pairs)) {
     bvec = c(bvec, total_pairs)
   }
-  
-  
+
+
   #! ub
   ub = rep(1, n_dec)
-  
+
   # sense
   sense = rep("L", length(table(rows_nbm)))
   #sense = c(sense, rep("G", length(table(rows_far_all))))
   #sense = c(sense, rep("E", length(table(rows_far_pairs))))
-  
+
   if (!is.null(far_covs)) {
     n_far_covs = ncol(far_covs)
     for (j in 1:n_far_covs) {
-      if (!is.null(far_groups)) { 
-        sense = c(sense, rep("G", 1)) 
+      if (!is.null(far_groups)) {
+        sense = c(sense, rep("G", 1))
       }
       if (!is.null(far_pairs) && all(rows_ind_far_pairs[[j]] != -1)) {
         sense = c(sense, rep("E", length(table(rows_ind_far_pairs[[j]]))))
       }
     }
   }
-  
+
   #sense = c(sense, rep("L", length(table(rows_near_all))))
   #sense = c(sense, rep("E", length(table(rows_near_pairs))))
-  
+
   if (!is.null(near_covs)) {
     n_near_covs = ncol(near_covs)
     for (j in 1:n_near_covs) {
-      if (!is.null(near_groups)) { 
-        sense = c(sense, rep("L", 1)) 
+      if (!is.null(near_groups)) {
+        sense = c(sense, rep("L", 1))
       }
       if (!is.null(near_pairs) && all(rows_ind_near_pairs[[j]] != -1)) {
         sense = c(sense, rep("E", length(table(rows_ind_near_pairs[[j]]))))
       }
     }
   }
-  
+
   sense = c(sense, rep("L", length(table(rows_mom))))
-  
+
   sense = c(sense, rep("L", length(table(rows_target))))
-  
+
   if (!is.null(exact_covs)) {
     sense = c(sense, rep("E", ncol(exact_covs)))
   }
@@ -609,105 +601,45 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
   sense = c(sense, rep("E", length(table(rows_fine))))
   sense = c(sense, rep(c("G", "L"), length(table(rows_near_fine))/2))
   sense = c(sense, rep("E", length(total_pairs)))
-  
+
   # var_type
   if (approximate == 1) {
     var_type = rep("C", n_dec)
   }
   else {
     var_type = rep("B", n_dec)
-  }	
-  
-  # Solve
-  if (solver=="cplex") {
-    #library("Rcplex")
-    if (requireNamespace('Rcplex', quietly = TRUE)) {
-      cat(format("  CPLEX optimizer is open..."), "\n")
-      ptm = proc.time()
-      out = Rcplex::Rcplex(cvec, Amat, bvec, ub = ub, sense = sense, vtype = var_type,
-                           control = list(trace = trace, round = round_cplex, tilim = t_max))
-      time = (proc.time()-ptm)[3]
-      
-      # Output
-      if (out$status==108) {
-        cat(format("  Error: time limit exceeded, no integer solution!"), "\n")
-        obj_val = NA
-        obj_dist_mat = NA
-        id_1 = NA
-        id_2 = NA
-        group_id = NA
-        time = NA
-      } else if (is.na(out$obj)) {
-        cat(format("  Error: problem infeasible!"), "\n")
-        obj_val = NA
-        obj_dist_mat = NA
-        id_1 = NA
-        id_2 = NA
-        group_id = NA
-        time = NA
-      }
-      
-      if (!is.na(out$obj)) {
-        cat(format("  Optimal matches found"), "\n")
-        
-        if (approximate == 1) {
-          rel = .relaxation_n(n_tot, out$xopt, dist_mat, subset_weight, "cplex", round_cplex, trace)
-          out$xopt = rel$sol
-          out$obj = rel$obj
-          time = time + rel$time
-        }
-        
-        i_ind = rep(1:(n_tot-1), (n_tot-1):1)
-        aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
-        j_ind = aux[lower.tri(aux)]
-        
-        group_1 = i_ind[out$xopt==1]
-        group_2 = j_ind[out$xopt==1]
-        max_groups = apply(cbind(group_1, group_2), 1, max)
-        
-        id_1 = group_1[max_groups<=n_tot]
-        id_2 = group_2[max_groups<=n_tot]
-        
-        #! Group identifier
-        group_id_1 = 1:(length(id_1))
-        group_id_2 = 1:(length(id_2))
-        group_id = c(group_id_1, group_id_2)
-        
-        obj_val = out$obj
-        obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$xopt)
-      }
-    } else {
-      stop('suggested package not installed')
-    }
   }
-  
-  if (solver == "highs"){
-    #library(highs)
-    cat(format("  HiGHS optimizer is open..."), "\n")
+
+  # Solve
+  #! HiGHS
+  if (solver == "highs") {
+
+    message("  HiGHS optimizer is open...")
+
     lhs = rep(-Inf, length(sense))
     rhs = rep(Inf, length(sense))
     lhs[sense == "G"] = bvec[sense == "G"]
     rhs[sense == "L"] = bvec[sense == "L"]
     lhs[sense == "E"] = bvec[sense == "E"]
     rhs[sense == "E"] = bvec[sense == "E"]
-    
+
     types = var_type
     types[types=="B"] = "I"
-    
-    cat(format("  Finding the optimal matches..."), "\n")
+
+    message("  Finding the optimal matches...")
     ptm = proc.time()
-    out = highs_solve(L = cvec,
-                      lower = 0,
-                      upper = ub,
-                      A = Amat,
-                      lhs = lhs,
-                      rhs = rhs,
-                      types = types,
-                      control = (highs_control(time_limit = t_max)))
-    
+    out = highs::highs_solve(L = cvec,
+                             lower = 0,
+                             upper = ub,
+                             A = Amat,
+                             lhs = lhs,
+                             rhs = rhs,
+                             types = types,
+                             control = highs::highs_control(time_limit = t_max))
+
     time = (proc.time()-ptm)[3]
     if (out$status == 8) {
-      cat(format("  Error: problem infeasible!"), "\n")
+      message("  Error: problem infeasible!")
       obj_val = NA
       obj_dist_mat = NA
       id_1 = NA
@@ -715,242 +647,311 @@ nmatch = function(dist_mat, subset_weight = NULL, total_pairs = NULL,
       group_id = NA
       time = NA
     }
-    else if (out$status == 7 | out$status == 13){
+    else if (out$status == 7 || out$status == 13){
       if (out$status == 7){
-        cat(format("  Optimal matches found"), "\n")
+        message("  Optimal matches found")
       }
       else if (out$status == 13){
-        cat(format("  Time limit reached!"), "\n")
+        message("  Time limit reached!")
       }
-      
+
       if (approximate == 1) {
         rel = .relaxation_n(n_tot, out$primal_solution, dist_mat, subset_weight, "highs", round_cplex, trace)
         out$primal_solution = rel$sol
         out$objval = rel$obj
         time = time + rel$time
       }
-      
-      i_ind = rep(1:(n_tot-1), (n_tot-1):1)  
+
+      i_ind = rep(1:(n_tot-1), (n_tot-1):1)
       aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
       j_ind = aux[lower.tri(aux)]
-      
+
       group_1 = i_ind[round(out$primal_solution, 1e-10)==1]
       group_2 = j_ind[round(out$primal_solution, 1e-10)==1]
       max_groups = apply(cbind(group_1, group_2), 1, max)
-      
+
       id_1 = group_1[max_groups<=n_tot]
       id_2 = group_2[max_groups<=n_tot]
-      
+
       #! Group identifier
       group_id_1 = 1:(length(id_1))
       group_id_2 = 1:(length(id_2))
       group_id = c(group_id_1, group_id_2)
-      
+
       obj_val = out$objval
       obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * (round(out$primal_solution, 1e-10) == 1))
     }
   }
-  
-  if (solver=="gurobi") {
-    #library("gurobi")
-    if (requireNamespace('gurobi', quietly = TRUE)) {
-      cat(format("  Gurobi optimizer is open..."), "\n")
-      model = list()
-      model$modelsense = 'min'
-      model$obj = cvec
-      model$A = Amat
-      model$sense = rep(NA, length(sense))
-      model$sense[sense=="E"] = '='
-      model$sense[sense=="L"] = '<='
-      model$sense[sense=="G"] = '>='
-      model$rhs = bvec
-      model$vtypes = var_type
-      model$ub = ub
-      
-      t_lim = list(TimeLimit = t_max, OutputFlag = trace)
-      
-      ptm = proc.time()
-      out = gurobi::gurobi(model, t_lim)
-      time = (proc.time()-ptm)[3]
-      
-      # Output
-      if (out$status == "INFEASIBLE") {
-        cat(format("  Error: problem infeasible!"), "\n")
-        obj_val = NA
-        obj_dist_mat = NA
-        id_1 = NA
-        id_2 = NA
-        group_id = NA
-        time = NA
-      }
-      
-      if (out$status ==  "OPTIMAL" || out$status == "TIME_LIMIT") {
-        if (out$status == "OPTIMAL") {
-          cat(format("  Optimal matches found"), "\n")
-        }
-        else {
-          cat(format("  Time limit reached, best suboptimal solution given"), "\n")
-        }
-        
-        if (approximate == 1) {
-          rel = .relaxation_n(n_tot, out$x, dist_mat, subset_weight, "gurobi", round_cplex, trace)
-          out$x = rel$sol
-          out$objval = rel$obj
-          time = time + rel$time
-        }
-        
-        i_ind = rep(1:(n_tot-1), (n_tot-1):1)  
-        aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
-        j_ind = aux[lower.tri(aux)]
-        
-        group_1 = i_ind[out$x==1]
-        group_2 = j_ind[out$x==1]
-        max_groups = apply(cbind(group_1, group_2), 1, max)
-        
-        id_1 = group_1[max_groups<=n_tot]
-        id_2 = group_2[max_groups<=n_tot]
-        
-        #! Group identifier
-        group_id_1 = 1:(length(id_1))
-        group_id_2 = 1:(length(id_2))
-        group_id = c(group_id_1, group_id_2)
-        
-        obj_val = out$objval
-        obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$x)
-      }
-    } else {
-      stop('suggested package not installed')
+
+  #! Gurobi
+  else if (solver == "gurobi") {
+    if (!requireNamespace('gurobi', quietly = TRUE)) {
+      stop('Required solver not installed')
     }
+
+    message("  Gurobi optimizer is open...")
+    model = list()
+    model$modelsense = 'min'
+    model$obj = cvec
+    model$A = Amat
+    model$sense = rep(NA, length(sense))
+    model$sense[sense=="E"] = '='
+    model$sense[sense=="L"] = '<='
+    model$sense[sense=="G"] = '>='
+    model$rhs = bvec
+    model$vtypes = var_type
+    model$ub = ub
+
+    t_lim = list(TimeLimit = t_max, OutputFlag = trace)
+
+    ptm = proc.time()
+    out = gurobi::gurobi(model, t_lim)
+    time = (proc.time()-ptm)[3]
+
+    # Output
+    if (out$status == "INFEASIBLE") {
+      message("  Error: problem infeasible!")
+      obj_val = NA
+      obj_dist_mat = NA
+      id_1 = NA
+      id_2 = NA
+      group_id = NA
+      time = NA
+    }
+
+    if (out$status ==  "OPTIMAL" || out$status == "TIME_LIMIT") {
+      if (out$status == "OPTIMAL") {
+        message("  Optimal matches found")
+      }
+      else {
+        message("  Time limit reached, best suboptimal solution given")
+      }
+
+      if (approximate == 1) {
+        rel = .relaxation_n(n_tot, out$x, dist_mat, subset_weight, "gurobi", round_cplex, trace)
+        out$x = rel$sol
+        out$objval = rel$obj
+        time = time + rel$time
+      }
+
+      i_ind = rep(1:(n_tot-1), (n_tot-1):1)
+      aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
+      j_ind = aux[lower.tri(aux)]
+
+      group_1 = i_ind[out$x==1]
+      group_2 = j_ind[out$x==1]
+      max_groups = apply(cbind(group_1, group_2), 1, max)
+
+      id_1 = group_1[max_groups<=n_tot]
+      id_2 = group_2[max_groups<=n_tot]
+
+      #! Group identifier
+      group_id_1 = 1:(length(id_1))
+      group_id_2 = 1:(length(id_2))
+      group_id = c(group_id_1, group_id_2)
+
+      obj_val = out$objval
+      obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$x)
+    }
+
   }
-  
-  if (solver=="glpk") {
-    #library("Rglpk")
-    if (requireNamespace('Rglpk', quietly = TRUE)) {
-      cat(format("  GLPK optimizer is open..."), "\n")
-      dir = rep(NA, length(sense))
-      dir[sense=="E"] = '=='
-      dir[sense=="L"] = '<='
-      dir[sense=="G"] = '>='
-      bound = list(lower = list(ind=c(1:length(ub)), val=rep(0,length(ub))),
-                   upper = list(ind=c(1:length(ub)), val=ub))
-      ptm = proc.time()
-      out = Rglpk::Rglpk_solve_LP(cvec, Amat, dir, bvec, bounds = bound, types = var_type, max = FALSE)
-      time = (proc.time()-ptm)[3]
-      
-      # Output
-      if (out$status!=0) {
-        cat(format("  Error: problem infeasible!"), "\n")
-        obj_val = NA
-        obj_dist_mat = NA
-        id_1 = NA
-        id_2 = NA
-        group_id = NA
-        time = NA
-      }
-      
-      if (out$status==0) {
-        cat(format("  Optimal matches found"), "\n")
-        
-        if (approximate == 1) {
-          rel = .relaxation_n(n_tot, out$solution, dist_mat, subset_weight, "glpk", round_cplex, trace)
-          out$solution = rel$sol
-          out$optimum = rel$obj
-          time = time + rel$time
-        }
-        
-        i_ind = rep(1:(n_tot-1), (n_tot-1):1)
-        aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
-        j_ind = aux[lower.tri(aux)]
-        
-        group_1 = i_ind[out$solution==1]
-        group_2 = j_ind[out$solution==1]
-        max_groups = apply(cbind(group_1, group_2), 1, max)
-        
-        id_1 = group_1[max_groups<=n_tot]
-        id_2 = group_2[max_groups<=n_tot]
-        
-        #! Group identifier
-        group_id_1 = 1:(length(id_1))
-        group_id_2 = 1:(length(id_2))
-        group_id = c(group_id_1, group_id_2)
-        
-        obj_val = out$optimum
-        obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$solution)
-      }
+
+  #! CPLEX
+  else if (solver == "cplex") {
+    if (!requireNamespace('Rcplex', quietly = TRUE)) {
+      stop('Required solver not installed')
     }
-    else {
-      stop('suggested package not installed')
+
+    message("  CPLEX optimizer is open...")
+    ptm = proc.time()
+    out = Rcplex::Rcplex(cvec, Amat, bvec, ub = ub, sense = sense, vtype = var_type,
+                         control = list(trace = trace, round = round_cplex, tilim = t_max))
+    time = (proc.time()-ptm)[3]
+
+    # Output
+    if (out$status==108) {
+      message("  Error: time limit exceeded, no integer solution!")
+      obj_val = NA
+      obj_dist_mat = NA
+      id_1 = NA
+      id_2 = NA
+      group_id = NA
+      time = NA
+    } else if (is.na(out$obj)) {
+      message("  Error: problem infeasible!")
+      obj_val = NA
+      obj_dist_mat = NA
+      id_1 = NA
+      id_2 = NA
+      group_id = NA
+      time = NA
     }
+
+    if (!is.na(out$obj)) {
+      message("  Optimal matches found")
+
+      if (approximate == 1) {
+        rel = .relaxation_n(n_tot, out$xopt, dist_mat, subset_weight, "cplex", round_cplex, trace)
+        out$xopt = rel$sol
+        out$obj = rel$obj
+        time = time + rel$time
+      }
+
+      i_ind = rep(1:(n_tot-1), (n_tot-1):1)
+      aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
+      j_ind = aux[lower.tri(aux)]
+
+      group_1 = i_ind[out$xopt==1]
+      group_2 = j_ind[out$xopt==1]
+      max_groups = apply(cbind(group_1, group_2), 1, max)
+
+      id_1 = group_1[max_groups<=n_tot]
+      id_2 = group_2[max_groups<=n_tot]
+
+      #! Group identifier
+      group_id_1 = 1:(length(id_1))
+      group_id_2 = 1:(length(id_2))
+      group_id = c(group_id_1, group_id_2)
+
+      obj_val = out$obj
+      obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$xopt)
+    }
+
   }
-  
-  if (solver=="symphony") {
-    #library("Rsymphony")
-    if (requireNamespace('Rsymphony', quietly = TRUE)) {
-      cat(format("  Symphony optimizer is open..."), "\n")
-      dir = rep(NA, length(sense))
-      dir[sense=="E"] = '=='
-      dir[sense=="L"] = '<='
-      dir[sense=="G"] = '>='
-      bound = list(lower = list(ind=c(1:length(ub)), val=rep(0,length(ub))),
-                   upper = list(ind=c(1:length(ub)), val=ub))
-      ptm = proc.time()
-      out = Rsymphony::Rsymphony_solve_LP(cvec, Amat, dir, bvec, bounds = bound, types = var_type, max = FALSE, time_limit = t_max)
-      time = (proc.time()-ptm)[3]
-      
-      # Output
-      if (out$status==228) {
-        cat(format("  Error: problem exceeded the time limit and no feasible solution is found!"), "\n")
-        obj_val = NA
-        obj_dist_mat = NA
-        id_1 = NA
-        id_2 = NA
-        group_id = NA
-        time = NA
-      }
-      else if (out$status!=0) {
-        cat(format("  Error: problem infeasible!"), "\n")
-        obj_val = NA
-        obj_dist_mat = NA
-        id_1 = NA
-        id_2 = NA
-        group_id = NA
-        time = NA
-      }
-      
-      if (out$status==0) {
-        cat(format("  Optimal matches found"), "\n")
-        
-        if (approximate == 1) {
-          rel = .relaxation_n(n_tot, out$solution, dist_mat, subset_weight, "symphony")
-          out$solution = rel$sol
-          out$objval = rel$obj
-          time = time + rel$time
-        }
-        
-        i_ind = rep(1:(n_tot-1), (n_tot-1):1)
-        aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
-        j_ind = aux[lower.tri(aux)]
-        
-        group_1 = i_ind[out$solution==1]
-        group_2 = j_ind[out$solution==1]
-        max_groups = apply(cbind(group_1, group_2), 1, max)
-        
-        id_1 = group_1[max_groups<=n_tot]
-        id_2 = group_2[max_groups<=n_tot]
-        
-        #! Group identifier
-        group_id_1 = 1:(length(id_1))
-        group_id_2 = 1:(length(id_2))
-        group_id = c(group_id_1, group_id_2)
-        
-        obj_val = out$objval
-        obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$solution)
-      }
-    } else {
-      stop('suggested package not installed')
+
+  #! GLPK
+  else if (solver == "glpk") {
+    if (!requireNamespace('Rglpk', quietly = TRUE)) {
+      stop('Required solver not installed')
     }
+
+    message("  GLPK optimizer is open...")
+    dir = rep(NA, length(sense))
+    dir[sense=="E"] = '=='
+    dir[sense=="L"] = '<='
+    dir[sense=="G"] = '>='
+    bound = list(lower = list(ind=c(1:length(ub)), val=rep(0,length(ub))),
+                 upper = list(ind=c(1:length(ub)), val=ub))
+    ptm = proc.time()
+    out = Rglpk::Rglpk_solve_LP(cvec, Amat, dir, bvec, bounds = bound, types = var_type, max = FALSE)
+    time = (proc.time()-ptm)[3]
+
+    # Output
+    if (out$status!=0) {
+      message("  Error: problem infeasible!")
+      obj_val = NA
+      obj_dist_mat = NA
+      id_1 = NA
+      id_2 = NA
+      group_id = NA
+      time = NA
+    }
+
+    if (out$status==0) {
+      message("  Optimal matches found")
+
+      if (approximate == 1) {
+        rel = .relaxation_n(n_tot, out$solution, dist_mat, subset_weight, "glpk", round_cplex, trace)
+        out$solution = rel$sol
+        out$optimum = rel$obj
+        time = time + rel$time
+      }
+
+      i_ind = rep(1:(n_tot-1), (n_tot-1):1)
+      aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
+      j_ind = aux[lower.tri(aux)]
+
+      group_1 = i_ind[out$solution==1]
+      group_2 = j_ind[out$solution==1]
+      max_groups = apply(cbind(group_1, group_2), 1, max)
+
+      id_1 = group_1[max_groups<=n_tot]
+      id_2 = group_2[max_groups<=n_tot]
+
+      #! Group identifier
+      group_id_1 = 1:(length(id_1))
+      group_id_2 = 1:(length(id_2))
+      group_id = c(group_id_1, group_id_2)
+
+      obj_val = out$optimum
+      obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$solution)
+    }
+
   }
-  
-  return = list(obj_total = obj_val, obj_dist_mat = obj_dist_mat, id_1 = id_1, id_2 = id_2, 
-                group_id = group_id,time = time)
+
+  #! Symphony
+  else if (solver == "symphony") {
+    if (!requireNamespace('Rsymphony', quietly = TRUE)) {
+      stop('Required solver not installed')
+    }
+
+    message("  Symphony optimizer is open...")
+    dir = rep(NA, length(sense))
+    dir[sense=="E"] = '=='
+    dir[sense=="L"] = '<='
+    dir[sense=="G"] = '>='
+    bound = list(lower = list(ind=c(1:length(ub)), val=rep(0,length(ub))),
+                 upper = list(ind=c(1:length(ub)), val=ub))
+    ptm = proc.time()
+    out = Rsymphony::Rsymphony_solve_LP(cvec, Amat, dir, bvec, bounds = bound, types = var_type, max = FALSE, time_limit = t_max)
+    time = (proc.time()-ptm)[3]
+
+    # Output
+    if (out$status==228) {
+      message("  Error: problem exceeded the time limit and no feasible solution is found!")
+      obj_val = NA
+      obj_dist_mat = NA
+      id_1 = NA
+      id_2 = NA
+      group_id = NA
+      time = NA
+    }
+    else if (out$status!=0) {
+      message("  Error: problem infeasible!")
+      obj_val = NA
+      obj_dist_mat = NA
+      id_1 = NA
+      id_2 = NA
+      group_id = NA
+      time = NA
+    }
+
+    if (out$status==0) {
+      message("  Optimal matches found")
+
+      if (approximate == 1) {
+        rel = .relaxation_n(n_tot, out$solution, dist_mat, subset_weight, "symphony")
+        out$solution = rel$sol
+        out$objval = rel$obj
+        time = time + rel$time
+      }
+
+      i_ind = rep(1:(n_tot-1), (n_tot-1):1)
+      aux = matrix(1:n_tot, nrow = n_tot, ncol = n_tot)
+      j_ind = aux[lower.tri(aux)]
+
+      group_1 = i_ind[out$solution==1]
+      group_2 = j_ind[out$solution==1]
+      max_groups = apply(cbind(group_1, group_2), 1, max)
+
+      id_1 = group_1[max_groups<=n_tot]
+      id_2 = group_2[max_groups<=n_tot]
+
+      #! Group identifier
+      group_id_1 = 1:(length(id_1))
+      group_id_2 = 1:(length(id_2))
+      group_id = c(group_id_1, group_id_2)
+
+      obj_val = out$objval
+      obj_dist_mat = sum(t(dist_mat)[lower.tri(dist_mat)] * out$solution)
+    }
+
+  }
+
+  list(obj_total = obj_val,
+       obj_dist_mat = obj_dist_mat,
+       id_1 = id_1,
+       id_2 = id_2,
+       group_id = group_id,
+       time = time)
 }
